# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 3
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# Original framework from Netrender
# Original Octane script from Enrico Cerica
# This script by Lionel Zamouth

import subprocess
import os
import time
import datetime
import mathutils
import bpy
import octanerender.settings
from octanerender.utils import *
from octanerender.export import *

class OctaneRenderEngine(bpy.types.RenderEngine):
	bl_idname = 'OCT_RENDER'
	bl_label = "Octane Render"
	bl_postprocess = False

	def render(self, scene):
		# Preset status to ERROR in case of plugin crash
		update_status(3,'Something wrong happened, please check console logs')

		# Accessors to both render environments
		octane_render = scene.octane_render
		blender_render = scene.render
				
		# Let's start
		start_time = datetime.datetime.now()
		log ('Plug-in starting in "%s" mode' % (octanerender.export_mode))
	
		# Get octane_render name
		baseName = fixName(octane_render.project_name)
		if baseName == '' or hasSpace(baseName):
			error('Project name is empty or contains spaces "%s"' % baseName)

		# Set and check project path
		basePath = absPath(octane_render.path)
		if not os.path.isdir(basePath) or hasSpace(basePath):
			error('Project directory is invalid or contains spaces "%s" ("%s")' % (octane_render.path,basePath))

		# Set and check image output path
		animPath = absPath(octane_render.image_output)
		if (octanerender.export_mode == 'anim' or octanerender.export_mode == 'fly') and (not os.path.isdir(animPath) or hasSpace(animPath)):
			error('Image output directory is invalid or contains spaces "%s" ("%s")' % (octane_render.path,animPath))
	
		# Set octane scene and obj filenames
		ocsFile = absPath('%s/%s.ocs' % (basePath,baseName))
		objFile = absPath('%s/%s.obj' % (basePath,baseName))
		mtlFile = absPath('%s/%s.mtl' % (basePath,baseName))
		log ('Output ocs: "%s"' % (ocsFile))
		log ('Output obj: "%s"' % (objFile))
		log ('Output mtl: "%s"' % (mtlFile))

		#Set the unit factor (meters, centimeters, inches, etc...
		unitFactor = 1
		unitFactor = {0:0.001,1:0.01,2:0.1,3:1,4:10,5:100,6:1000,7:0.0254,8:0.3048,9:0.9144,10:201.168,11:1609.344}[int(octane_render.unit_size)]
		log ('Unit Factor (rescaling): %.4f' % (unitFactor))
	
		first_frame = True
		while True:
			log ('Rendering frame #%d' % (scene.frame_current))
			# Export only first frame if fly mode, otherwise all frames
			if octanerender.export_mode != 'fly' or first_frame == True:
				obj_list = obj_export(scene, scene.frame_current)
				mtl_list = write_obj(objFile, mtlFile, obj_list, scene, unitFactor)
				if octane_render.export_materials:
					write_mtl(mtlFile, mtl_list, scene, octane_render.export_copy_images)
				first_frame = False

			# Return to blender if export mode
			if octanerender.export_mode=='export':
				break

			axes = ['x','y','z']
			paramDsp=''

			command_args = []
			# Check Octane binary
			exeFile = absPath(octane_render.binary)
			if not os.path.isfile(exeFile):
				error('Invalid Octane binary file "%s" ("%s")' % (octane_render.binary,exeFile))
			command_args.append(exeFile)

			# Set mesh object to use
			command_args.append('-m')
			command_args.append('%s.obj' % (baseName))

			# Manage Camera settings
			if octane_render.export_camera:

				# Check if we have a valid camera
				camCAM = bpy.data.cameras.get(octane_render.active_camera)
				if not camCAM:
					error('Invalid camera object or camera not selected: <%s>' % (octane_render.active_camera))
				log ('Using camera : <%s>' % (camCAM.name))
				camOBJ = bpy.data.objects.get(camCAM.name)


				# Set Field of View
				fov = camCAM.angle*180.0/3.1415926536
				command_args.append('--cam-fov')
				command_args.append('%f' % (fov))
				log ('field of View: %f' % (fov))

				# Manage Lens Aperture
				if octane_render.lens_aperture:
					command_args.append('--cam-aperture')
					command_args.append('%f' % (octane_render.lens_aperture_value))
					log ('Lens aperture: %f' % (octane_render.lens_aperture_value))

				# Manage Focal Depth / Depth of Field
				if octane_render.focal_depth == True:
					fd = 100.0
					# TOGGLE
					if octanerender.Supported == True:
							fd = octane_render.focal_depth_value
					else:
						log('Using Depth of Field from : %s' % (octane_render.active_camera))
						if camCAM.dof_object:
							tarOBJ = bpy.data.objects.get(camCAM.dof_object.name)
							if camCAM.dof_object.name == octane_render.active_camera:
								error('Narcissic camera...')
							fd = (tarOBJ.location - camOBJ.location).magnitude
							log ('Using DoF with distance to <%s> : %f * unitSize' % (tarOBJ.name,fd))
						else:
							fd = camCAM.dof_distance
							if fd < 0.00001:
								fd = 0
							log ('Using DoF from blender camera: %f * unitSize' % (fd))
					fd *= unitFactor
					# Ignore DoF of 0
					if fd > 0:
						command_args.append('--cam-focaldepth')
						command_args.append('%f' % (fd))

				# Set camera position and target
				matrix = camOBJ.matrix_world
				if octane_render.export_ROTX90 == True:
					position = rotate90x(matrix_vect(matrix,[0.0,0.0,0.0,1.0]))
					target = rotate90x(matrix_vect(matrix,[0.0,0.0,-1.0,1.0]))
					up = rotate90x(matrix_vect(matrix,[0.0,1.0,0.0,0.0]))
				else:
					position = matrix_vect(matrix,[0.0,0.0,0.0,1.0])
					target = matrix_vect(matrix,[0.0,0.0,-1.0,1.0])
					up = matrix_vect(matrix,[0.0,1.0,0.0,0.0])
				for i in range(3):
					command_args.append('--cam-pos-%s' % (axes[i]))
					command_args.append('%f' % (position[i]*unitFactor))
					if i==0:paramDsp=''
					paramDsp+= '%s %f ' % (axes[i],position[i]*unitFactor)
				log ('Camera position: %s' % (paramDsp))
				for i in range(3):
					command_args.append('--cam-target-%s' % (axes[i]))
					command_args.append('%f' % (target[i]*unitFactor))
					if i==0:paramDsp=''
					paramDsp+= '%s %f ' % (axes[i],target[i]*unitFactor)
				log ('Camera target: %s' % (paramDsp))
				for i in range(3):
					command_args.append('--cam-up-%s' % (axes[i]))
					command_args.append('%f' % (up[i]*unitFactor))
					if i==0:paramDsp=''
					paramDsp+= '%s %f ' % (axes[i],up[i]*unitFactor)
				log ('Camera up: %s' % (paramDsp))

				# Manage camera Motion
				if octane_render.camera_motion:
					currentFrame=scene.frame_current
					# Interpolate on next frame
					if octane_render.interpolate_frame=='0': 
						bpy.context.scene.frame_set(currentFrame + 1)
						log ('Motion interploate on next frame')
					# Interpolate on previous frame
					elif currentFrame>1:
						bpy.context.scene.frame_set(currentFrame - 1)
						log ('Motion interpolate on previous frame')

					matrix = camOBJ.matrix_world
					if octane_render.export_ROTX90 == True:
						position = rotate90x(matrix_vect(matrix,[0.0,0.0,0.0,1.0]))
						target = rotate90x(matrix_vect(matrix,[0.0,0.0,-1.0,1.0]))
						up = rotate90x(matrix_vect(matrix,[0.0,1.0,0.0,0.0]))
					else:
						position = matrix_vect(matrix,[0.0,0.0,0.0,1.0])
						target = matrix_vect(matrix,[0.0,0.0,-1.0,1.0])
						up = matrix_vect(matrix,[0.0,1.0,0.0,0.0])
					#matrix = bpy.context.scene.camera.matrix_world
					#position = rotate90x(matrix_vect(matrix,[0.0,0.0,0.0,1.0]))
					#target = rotate90x(matrix_vect(matrix,[0.0,0.0,-1.0,1.0]))
					up = rotate90x(matrix_vect(matrix,[0.0,1.0,0.0,0.0]))

					for i in range(3):
						command_args.append('--cam-motion-pos-%s' % (axes[i]))
						command_args.append('%f' % (position[i]*unitFactor))
						if i==0:paramDsp=''
						paramDsp+= '%s %f ' % (axes[i],position[i]*unitFactor)
					log ('Camera motion position: %s' % (paramDsp))
					for i in range(3):
						command_args.append('--cam-motion-target-%s' % (axes[i]))
						command_args.append('%f' % (target[i]*unitFactor))
						if i==0:paramDsp=''
						paramDsp+= '%s %f ' % (axes[i],target[i]*unitFactor)
					log ('Camera motion target: %s' % (paramDsp))
					for i in range(3):
						command_args.append('--cam-motion-up-%s' % (axes[i]))
						command_args.append('%f' % (up[i]*unitFactor))
						if i==0:paramDsp=''
						paramDsp+= '%s %f ' % (axes[i],up[i]*unitFactor)
					log ('Camera motion up: %s' % (paramDsp))
					bpy.context.scene.frame_set(currentFrame)
				
			# Manage daylight option
			light_param=''
			if octane_render.export_sundir:
				displayError=False
				try:
					light = bpy.data.objects[octane_render.active_light]
				except:
					displayError=True
				if octane_render.active_light=='' or displayError:
					error("Invalid sun light object or sun light not selected")
				else:
					log ('Using light : %s' % (light.name))
					matrix = mathutils.Matrix(light.matrix_world.copy()).invert()
					if octane_render.export_ROTX90 == True:
						matrix = rotate90x(matrix)
			
					for i in range(3):
						command_args.append('--daylight-sundir-%s' % (axes[i]))
						command_args.append('%f' % (matrix[i][2]*unitFactor))
						if i==0:paramDsp=''
						paramDsp+= '%s %f ' % (axes[i],matrix[i][2]*unitFactor)
					log ('Sun direction: %s' % (paramDsp))

			projectFileAlreadyExists = False
			if os.path.isfile(ocsFile):
					projectFileAlreadyExists=True
					log ('Project file already exists')

			#Set the exit after rendering flag (for animation)
			if octanerender.export_mode=='anim' or octanerender.export_mode=='fly':
				command_args.append('-e')
				command_args.append('-q')
		
			# Manage the resolution option
			x = blender_render.resolution_x
			y = blender_render.resolution_y
			if  octane_render.resolution:
				resSize = blender_render.resolution_percentage/100
				log ('Resolution factor : %d%%' % (resSize*100))
				command_args.append('--film-width')
				command_args.append('%i' % (x*resSize))
				command_args.append('--film-height')
				command_args.append('%i' % (y*resSize))
		
			#Set the GPUs option
			if octane_render.GPU_selector:
				for val in octane_render.GPU_use_list.split(' '):
						command_args.append('-g')
						command_args.append('%s' % (val))
				log ('GPUs to use: %s' % (octane_render.GPU_use_list))

			# Give a png output if in anim or fly mode
			if octanerender.export_mode=='anim' or octanerender.export_mode=='fly':
				pngFile = absPath('%s/%s_%06d.png' %(animPath,baseName,scene.frame_current))
				command_args.append('--output-png')
				command_args.append(pngFile)

			# Set samples to render if export_mode is 'anim' or 'fly'
			if octanerender.export_mode=='anim' or octanerender.export_mode=='fly':
				command_args.append('-s')
				command_args.append('%d' % (octane_render.export_samples_per_image))
				log ('Samples per image: %d' % (octane_render.export_samples_per_image))

			# Check if ocs needs to be overwritten or obj relinked
			if projectFileAlreadyExists and not octane_render.replace_project:
				if octane_render.relink_obj == True:
					log ('Project exists and obj will be relinked')
					command_args.append('-r')
					command_args.append(objFile)
				else:
					log ('Project exists and obj will NOT be relinked')
					#command_args.append('-l')
					#command_args.append(objFile)
					#command_args.append('-n')
			else:
				log ('Create or replace project')
				command_args.append('-l')
				command_args.append(objFile)
				command_args.append('-n')
			# Last argument: ocs file
			command_args.append(ocsFile)

			# Let's start the magic!
			log ('Launching Octane: {}'.format(command_args))
			octane_process = subprocess.Popen(command_args,executable=exeFile)

			# Return immediately if not in anim or fly mode
			if octanerender.export_mode != 'anim' and octanerender.export_mode != 'fly':
				break
				
			# Wait for OCT , convert and load image
			self.update_stats("", "Octane: Batch Rendering frame# {} (see console for progress), please wait...".format(scene.frame_current))
			# Hold the render results window hostage until OCT returns...
			log ('Frame #%d sent to Octane Render' % (scene.frame_current))
			bpy.context.scene.frame_set(scene.frame_current + scene.frame_step)
			if scene.frame_current > scene.frame_end:
				break
			#log ('Next frame: %d' % (scene.frame_current))
			while True:
				if octane_process.poll() != None:
					result = self.begin_result(0, 0, x, y)
					try:
						log ('Load image from file: {}'.format(pngfile))
						result.layers[0].load_from_file('{}'.format(pngfile))
					except:
						pass

					self.end_result(result)
					break
					
				if self.test_break():
					try:
						octane_process.terminate()
					except:
						pass
					break
					
				time.sleep(1)

		# Yes, we made it!
		update_status(0,'Completed in %s' % elapsed_long(start_time))

