# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 3
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# Original framework from Netrender
# Original Octane script from Enrico Cerica
# This script by Lionel Zamouth

import bpy
import octanerender.settings
from octanerender.utils import *
from octanerender.labels import getLabel

def base_poll(cls, context):
	rd = context.scene.render
	return (rd.use_game_engine==False) and (rd.engine in cls.COMPAT_ENGINES)

# Main Octane Panel
class OctaneRenderButtonsPanel():
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "render"
	# COMPAT_ENGINES must be defined in each subclass, external engines can add themselves here

	@classmethod
	def poll(cls, context):
		rd = context.scene.render
		return rd.engine == 'OCT_RENDER' and rd.use_game_engine == False

# Octane Render Plugin Panel
class RENDER_OCT_render_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane Unsupported Plug-in " + octanerender.Version
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='SEQUENCE')

	def draw(self, context):
		layout = self.layout
		scene = context.scene
		octane_render = scene.octane_render
		#bpy.ops.aspect.fix() 

		#col = layout.column_flow(columns=2,align=True)
		layout.prop(octane_render, "panel_mode", expand=True)
		#layout.template_list(octane_render, "panel_mode", active_data="A", rows=2, expand=False)
		if octane_render.panel_mode == "MODE_EXPORT":
			layout.operator("button_export",    icon='SCENE_DATA')
		elif octane_render.panel_mode == "MODE_RENDER":
			layout.operator("button_render",    icon='RENDER_STILL')
		else:
			if octane_render.panel_mode == "MODE_ANIM":
				layout.operator("button_animation", icon='RENDER_ANIMATION')
			else:
				layout.operator("button_fly", icon='CAMERA_DATA')

			row = layout.row()
			col = row.column()
			col.prop(octane_render, "image_output")

			row = layout.row()
			col = row.column()
			col.prop(octane_render, "export_samples_per_image")
			col = row.column()
			col.operator_menu_enum('menu_export_presets','export_samples_presets','Samples Presets')

# Dimensions Panel
class RENDER_OCT_dimensions(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_idname = "Dimensions"
	bl_label = "Dimensions"
	COMPAT_ENGINES = {'OCT_RENDER'}

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='SEQUENCE')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octane_render = scene.octane_render
		rd = scene.render

		row = layout.row(align=True)
		row.menu("RENDER_MT_presets", text=bpy.types.RENDER_MT_presets.bl_label)
		row.operator("render.preset_add", text="", icon="ZOOMIN")
		row.operator("render.preset_add", text="", icon="ZOOMOUT").remove_active = True

		split = layout.split()

		col = split.column()
		sub = col.column(align=True)
		sub.label(text="Resolution:")
		sub.prop(rd, "resolution_x", text="X")
		sub.prop(rd, "resolution_y", text="Y")
		sub.prop(rd, "resolution_percentage", text="")

		col = split.column()
		sub = col.column(align=True)
		sub.label(text="Frame Range:")
		sub.prop(scene, "frame_start", text="Start")
		sub.prop(scene, "frame_end", text="End")
		sub.prop(scene, "frame_step", text="Step")

# Octane Project Settings Panel
class RENDER_OCT_project_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane Project Settings"
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='URL')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octane_render = scene.octane_render

		if octane_render.path == "":
			layout.prop(octane_render, "path",icon='ERROR')
		else:
			layout.prop(octane_render, "path")

		if octane_render.project_name == "":
			layout.prop(octane_render, "project_name",icon='ERROR')
		else:
			layout.prop(octane_render, "project_name")
		layout.prop(octane_render, "unit_size")

# Octane Environment Settings Panel
class RENDER_OCT_environment_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane Environment Settings"
	#bl_options = {'DEFAULT_CLOSED'}
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='WORLD')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octane_render = scene.octane_render

		layout.prop(octane_render, "replace_project")
		if octane_render.replace_project == False:
			layout.prop(octane_render, "relink_obj")

		# Camera
		layout.separator()
		row = layout.row()
		row.label(text="Camera Properties", icon='SCENE')
		layout.prop(octane_render, "export_camera")
		if octane_render.export_camera:
			row = layout.row()
			split = row.split(percentage=0.75)
			col = split.column()
			if octane_render.active_camera == "":
				col.prop(octane_render, "active_camera",icon='ERROR')
			else:
				col.prop(octane_render, "active_camera")
			col = split.column()
			col.operator("button_select_camera",icon='SCENE')

			#row = layout.row()
			layout.prop(octane_render, "lens_aperture")
			if octane_render.lens_aperture:
				row = layout.row()
				split = row.split(percentage=0.75)
				col = split.column()
				col.prop(octane_render, "lens_aperture_value")
				col = split.column()
				col.prop(octane_render, "lens_aperture_curve")

			#row = layout.row()
			layout.prop(octane_render, "focal_depth")

			#row = layout.row()
			layout.prop(octane_render, "camera_motion")
			if octane_render.camera_motion:
				row = layout.row()
				split = row.split(percentage=0.75)
				col = split.column()
				col.prop(octane_render, "interpolate_frame")
			
		# Day light environment
		layout.separator()
		row = layout.row()
		row.label(text="Daylight Environment", icon='LAMP_SUN')
		layout.prop(octane_render, "export_sundir")
		if octane_render.export_sundir:
			row = layout.row()
			split = row.split(percentage=0.75)
			col = split.column()
			if octane_render.active_light == "":
				col.prop(octane_render, "active_light",icon='ERROR')
			else:
				col.prop(octane_render, "active_light")
			col = split.column()
			col.operator("button_select_light_source", icon='LAMP_SUN')

# Octane Export Settings
class RENDER_OCT_export_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane Export Settings"
	bl_options = {'DEFAULT_CLOSED'}
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='SCRIPTWIN')

	def draw(self, context):
		layout = self.layout

		scene = context.scene
		octane_render = scene.octane_render

		# Context
		row = layout.row()
		row.label(text=" Context",icon='GROUP')
		row = layout.row()
		split = row.split(percentage=0.50)
		col = split.column()
		col.prop(octane_render, "export_sel_only")
		col = split.column()
		col.prop(octane_render, "export_remove_hidden")
		
		# Output options
		layout.separator()
		row = layout.row()
		row.label(text=" Output Options",icon='MOD_MESHDEFORM')
		row = layout.row()
		split = row.split(percentage=0.33)

		col = split.column()
		col.prop(octane_render, "export_apply_modifiers")
		col = split.column()
		col.prop(octane_render, "export_ROTX90")
		col = split.column()
		col.prop(octane_render, "export_copy_images")

		layout.separator()
		row = layout.row()
		row.label(text=" Export Options",icon='SETTINGS')

		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(octane_render, "export_edges")
		col = split.column()
		col.prop(octane_render, "export_tri")
		col = split.column()
		col.prop(octane_render, "export_materials")
		col = split.column()
		col.prop(octane_render, "export_UV")

		row = layout.row()
		split = row.split(percentage=0.25)
		col = split.column()
		col.prop(octane_render, "export_normals")
		col = split.column()
		col.prop(octane_render, "export_HQ")
		col = split.column()
		col.prop(octane_render, "export_polygroups")
		col = split.column()
		col.prop(octane_render, "export_curves_as_nurbs")

# Octane System Settings
class RENDER_OCT_system_settings(OctaneRenderButtonsPanel, bpy.types.Panel):
	bl_label = "Octane System Settings"
	COMPAT_ENGINES = {'OCT_RENDER'}    

	@classmethod
	def poll(cls, context):
		return super().poll(context)

	def draw_header(self, context):
		layout = self.layout
		layout.label(text="", icon='RADIO')

	def draw(self, context):
		layout = self.layout
		scene = context.scene
		octane_render = scene.octane_render
		if octane_render.binary == "":
			layout.prop(octane_render, "binary",icon='ERROR')
		else:
			layout.prop(octane_render, "binary")

		row = layout.row()
		split = row.split(percentage=0.5)
		col = split.column()
		col.prop(octane_render, "GPU_selector")
		if octane_render.GPU_selector:
			col = split.column()
			col.prop(octane_render, "GPU_use_list")
		layout.prop(octane_render, "resolution")
		layout.prop(octane_render, "verbose")
		if octane_render.verbose == True:
			if octanerender.Verbose == False:
				octanerender.Verbose = True
				log('Turning verbose mode on')
			octanerender.Verbose = True
		else:
			if octanerender.Verbose == True:
				log('Turning verbose mode off')
			octanerender.Verbose = False


